"""contains some default parameters of the application."""

SMARTPHONE_ID = "pymultiMATIC"
"""Default smartphone id."""

SENSO = "SYSTEMCONTROL_SENSO"
MULTIMATIC = "SYSTEMCONTROL_MULTIMATIC"
"""Applications"""
